//V1 - initial release

#include <Wire.h> //for Chronodot

// Display output pin assignments
#define MTEN 	Display1=Display1 | (1<<0)  
#define HALF	Display1=Display1 | (1<<1)
#define QUARTER	Display1=Display1 | (1<<2)
#define TWENTY	Display1=Display1 | (1<<3)
#define MFIVE	Display1=Display1 | (1<<4)
#define ITIS	Display1=Display1 | (1<<5)
#define PAST	Display1=Display1 | (1<<6)
#define UNUSED1	Display1=Display1 | (1<<7)

#define TO	Display2=Display2 | (1<<0)
#define ONE	Display2=Display2 | (1<<1)
#define TWO	Display2=Display2 | (1<<2)
#define THREE	Display2=Display2 | (1<<3)
#define FOUR	Display2=Display2 | (1<<4)
#define HFIVE	Display2=Display2 | (1<<5)
#define SIX	Display2=Display2 | (1<<6)
#define UNUSED2	Display2=Display2 | (1<<7)

#define SEVEN	Display3=Display3 | (1<<0)
#define EIGHT	Display3=Display3 | (1<<1)
#define NINE	Display3=Display3 | (1<<2)
#define HTEN	Display3=Display3 | (1<<3)
#define ELEVEN	Display3=Display3 | (1<<4)
#define TWELVE	Display3=Display3 | (1<<5)
#define OCLOCK  Display3=Display3 | (1<<6)
#define UNUSED3	Display3=Display3 | (1<<7)

//force time to start at xx:xx:xx
//int  hour=9, minute=32, second=00;

int second; //00-59;
int minute; //00-59;
int hh24; //background time in 24hr mode for RTC
int hh; //12 hour mode for clock display

static unsigned long msTick =0;  // the number of Millisecond Ticks since we last incremented the second counter
char Display1=0, Display2=0, Display3=0;

// hardware constants
int LEDClockPin = 6;
int LEDDataPin = 7;
int LEDStrobePin = 8;
int FWDButtonPin = 4;
int REVButtonPin = 5;
int EnablePin = 9; //enables the shift regisers, ideally this should have been done at the hardware level

void setup() {

  Wire.begin();

  ////////////////////////////////
  /*
  second = 00;
   minute = 03;
   hour = 6;
   set_time();//just set the time once on the chronodot
  */
  ///////////////////////////////

  // initialise the hardware	
  // initialize the appropriate pins as outputs:
  pinMode(LEDClockPin, OUTPUT); 
  pinMode(LEDDataPin, OUTPUT); 
  pinMode(LEDStrobePin, OUTPUT); 
  pinMode(FWDButtonPin, INPUT); 
  pinMode(REVButtonPin, INPUT); 
  pinMode(EnablePin, OUTPUT); 

  //enable the shift registers
  digitalWrite(EnablePin, HIGH);
  
  Serial.begin(9600);   // setup the serial port to 9600 baud

  msTick=millis();      // Initialise the msTick counter
  
  selftest();      // run self test (displayTime is then called from selftest method)

  initChrono();//just get the time once on the chronodot
}

//turn off all the LEDs
void ledsoff(void) {
  Display1=0;
  Display2=0;
  Display3=0;
}

//write LEDs to the display
void WriteLEDs(void) {
  // Now we write the actual values to the hardware
  shiftOut(LEDDataPin, LEDClockPin, MSBFIRST, Display3);
  shiftOut(LEDDataPin, LEDClockPin, MSBFIRST, Display2);
  shiftOut(LEDDataPin, LEDClockPin, MSBFIRST, Display1);
  digitalWrite(LEDStrobePin,HIGH);
  delay(2);
  digitalWrite(LEDStrobePin,LOW);    
}

//self test
void selftest(void){

  Serial.print("Running self test...");
  ledsoff();

  //flash LEDs
  for(int i=0; i<3; i++)
  {
    Display1=255; 
    Display2=255; 
    Display3=255;
    WriteLEDs(); 
    delay(200);
    ledsoff(); 
    WriteLEDs();
    delay(200);
  }

  //run through all LEDs
  ledsoff(); 
  ITIS; 
  WriteLEDs(); 
  Serial.print("21");
  delay(200); 

  ledsoff(); 
  QUARTER; 
  WriteLEDs(); 
  Serial.print("20");
  delay(200); 

  ledsoff(); 
  TWENTY; 
  WriteLEDs(); 
  Serial.print("19");
  delay(200); 

  ledsoff(); 
  MFIVE; 
  WriteLEDs(); 
  Serial.print("18");
  delay(200); 

  ledsoff(); 
  HALF; 
  WriteLEDs(); 
  Serial.print("17");
  delay(200); 

  ledsoff(); 
  MTEN; 
  WriteLEDs(); 
  Serial.print("16");
  delay(200); 

  ledsoff(); 
  TO; 
  WriteLEDs(); 
  Serial.print("15");
  delay(200); 

  ledsoff(); 
  PAST; 
  WriteLEDs(); 
  Serial.print("14");
  delay(200); 

  ledsoff(); 
  NINE; 
  WriteLEDs(); 
  Serial.print("13");
  delay(200); 

  ledsoff(); 
  ONE; 
  WriteLEDs(); 
  Serial.print("12");
  delay(200); 

  ledsoff(); 
  SIX; 
  WriteLEDs(); 
  Serial.print("11");
  delay(200); 

  ledsoff(); 
  THREE; 
  WriteLEDs(); 
  Serial.print("10");
  delay(200); 

  ledsoff(); 
  FOUR; 
  WriteLEDs(); 
  Serial.print("9");
  delay(200); 

  ledsoff(); 
  HFIVE; 
  WriteLEDs(); 
  Serial.print("8");
  delay(200); 

  ledsoff(); 
  TWO; 
  WriteLEDs(); 
  Serial.print("7");
  delay(200); 

  ledsoff();
  EIGHT; 
  WriteLEDs(); 
  Serial.print("6");
  delay(200); 

  ledsoff(); 
  ELEVEN; 
  WriteLEDs(); 
  Serial.print("5");
  delay(200); 

  ledsoff(); 
  SEVEN; 
  WriteLEDs(); 
  Serial.print("4");
  delay(200); 

  ledsoff(); 
  TWELVE; 
  WriteLEDs(); 
  Serial.print("3");
  delay(200); 

  ledsoff(); 
  HTEN; 
  WriteLEDs(); 
  Serial.print("2");
  delay(200); 

  ledsoff(); 
  OCLOCK; 
  WriteLEDs(); 
  Serial.print("1");
  delay(200); 

  Serial.print("Display the time...");
  displaytime();
}


//construct the time to be displayed
void displaytime(void){

  get_time(); //from chronodot

  //pass RTC values into a new variable -
  //this way one variable stores time in 24 hour mode and another that can be converted to 12 hour mode
  //time is monitored 'in the background' in 24 hour mode using the RTC, but is displayed in an adjusted 12-hr format
  hh = hh24;

  //convert to 12 hr mode for display
  if (hh24 < 1) {
    hh = 12;
  }

  if ((hh24 > 12) && (hh24 < 24)) {
    hh = hh24 - 12;
  }

  // start by clearing the display
  ledsoff();

  // Now, turn on the "It is" leds
  ITIS;
  Serial.print("It is ");

  // now we display the appropriate minute counter
  if ((minute>4) && (minute<10)) { 
    MFIVE; 
    Serial.print("Five Minutes ");
  } 
  if ((minute>9) && (minute<15)) { 
    MTEN; 
    Serial.print("Ten Minutes ");
  }
  if ((minute>14) && (minute<20)) {
    QUARTER; 
    Serial.print("Quarter ");
  }
  if ((minute>19) && (minute<25)) { 
    TWENTY; 
    Serial.print("Twenty Minutes ");
  }
  if ((minute>24) && (minute<30)) { 
    TWENTY; 
    MFIVE; 
    Serial.print("Twenty Five Minutes ");
  }  
  if ((minute>29) && (minute<35)) {
    HALF;
    Serial.print("Half ");
  }
  if ((minute>34) && (minute<40)) { 
    TWENTY; 
    MFIVE; 
    Serial.print("Twenty Five Minutes ");
  }  
  if ((minute>39) && (minute<45)) { 
    TWENTY; 
    Serial.print("Twenty Minutes ");
  }
  if ((minute>44) && (minute<50)) {
    QUARTER; 
    Serial.print("Quarter ");
  }
  if ((minute>49) && (minute<55)) { 
    MTEN; 
    Serial.print("Ten Minutes ");
  } 
  if (minute>54) { 
    MFIVE; 
    Serial.print("Five Minutes ");
  }

  if ((minute <5))
  {
    switch (hh) {
    case 1: 
      ONE; 
      Serial.print("One ");
      break;
    case 2: 
      TWO; 
      Serial.print("Two ");
      break;
    case 3: 
      THREE; 
      Serial.print("Three ");
      break;
    case 4: 
      FOUR; 
      Serial.print("Four ");
      break;
    case 5: 
      HFIVE; 
      Serial.print("Five ");
      break;
    case 6: 
      SIX; 
      Serial.print("Six ");
      break;
    case 7: 
      SEVEN; 
      Serial.print("Seven ");
      break;
    case 8: 
      EIGHT; 
      Serial.print("Eight ");
      break;
    case 9: 
      NINE; 
      Serial.print("Nine ");
      break;
    case 10: 
      HTEN; 
      Serial.print("Ten ");
      break;
    case 11: 
      ELEVEN; 
      Serial.print("Eleven ");
      break;
    case 12: 
      TWELVE; 
      Serial.print("Twelve ");
      break;
    }
    OCLOCK;
    Serial.println("O'Clock");
  }
  else
    if ((minute < 35) && (minute >4))
    {
      PAST;
      Serial.print("Past ");
      switch (hh) {
      case 1: 
        ONE; 
        Serial.println("One ");
        break;
      case 2: 
        TWO; 
        Serial.println("Two ");
        break;
      case 3: 
        THREE; 
        Serial.println("Three ");
        break;
      case 4: 
        FOUR; 
        Serial.println("Four ");
        break;
      case 5: 
        HFIVE; 
        Serial.println("Five ");
        break;
      case 6: 
        SIX; 
        Serial.println("Six ");
        break;
      case 7: 
        SEVEN; 
        Serial.println("Seven ");
        break;
      case 8: 
        EIGHT; 
        Serial.println("Eight ");
        break;
      case 9: 
        NINE; 
        Serial.println("Nine ");
        break;
      case 10: 
        HTEN; 
        Serial.println("Ten ");
        break;
      case 11: 
        ELEVEN; 
        Serial.println("Eleven ");
        break;
      case 12: 
        TWELVE; 
        Serial.println("Twelve ");
        break;
      }
    }
    else
    {
      // if we are greater than 34 minutes past the hour then display
      // the next hour, as we will be displaying a 'to' sign
      TO;
      Serial.print("To ");
      switch (hh) {
      case 1: 
        TWO; 
        Serial.println("Two ");
        break;
      case 2: 
        THREE; 
        Serial.println("Three ");
        break;
      case 3: 
        FOUR; 
        Serial.println("Four ");
        break;
      case 4: 
        HFIVE; 
        Serial.println("Five ");
        break;
      case 5: 
        SIX; 
        Serial.println("Six ");
        break;
      case 6: 
        SEVEN; 
        Serial.println("Seven ");
        break;
      case 7: 
        EIGHT; 
        Serial.println("Eight ");
        break;
      case 8: 
        NINE; 
        Serial.println("Nine ");
        break;
      case 9: 
        HTEN; 
        Serial.println("Ten ");
        break;
      case 10: 
        ELEVEN; 
        Serial.println("Eleven ");
        break;
      case 11: 
        TWELVE; 
        Serial.println("Twelve ");
        break;
      case 12: 
        ONE; 
        Serial.println("One ");
        break;
      }
    }

  WriteLEDs();

}

// increment the time counters keeping care to rollover as required
void incrementtime(void) {
  
  second = 0;
  if (++minute >= 60) {
    minute = 0;
    if (++hh24 > 23) {
      hh24 = 0;  
    }
  }  
  
  // debug outputs
  Serial.println();
  Serial.print(hh);
  Serial.print(",");
  Serial.print(minute);
  Serial.print(",");
  Serial.println(second);
}


//main loop
void loop(void) {
  // heart of the timer - keep looking at the millisecond timer on the Arduino
  // and increment the seconds counter every 1000 ms
  if ( millis() - msTick > 999) {
    msTick = millis();
    second++;   
  }

  //test to see if we need to increment the time counters
  if (second == 60) 
  {
    incrementtime();
    displaytime();
  }

  // test to see if a forward button was pressed
  if ((digitalRead(FWDButtonPin) == 0 ) && second != 1) {
    // the forward button is down
    // and it has been more than one second since we
    // last looked
  
    delay(200); //do I even need this? Trying to debounce the switch so that if you press both together it'll run the self test and not change the time
    
    minute = (((minute/5)*5) +5); 
    second = 0;
    incrementtime();
    second++;  // Increment the second counter to ensure that the name
    // flash doesnt happen when setting time

    set_time();  //send time to Chronodot

    displaytime();
  }

  // test to see if the back button was pressed
  if ((digitalRead(REVButtonPin) == 0 ) && second != 1) {
   
    delay(200); //do I even need this. Trying to debounce the switch so that if you press both together it'll run the self test and not change the time
    
    minute = (((minute/5)*5) -5); 
    second = 0; // decrement the minute counter
    if (minute < 0) { 
      minute = 55; 
      if (--hh24 <0) hh24 = 12;
    } 
    incrementtime();
    second++;  // Increment the second counter to ensure that the name
    // flash doesnt happen when setting time  

    set_time();  //send time to Chronodot

    displaytime();
  }

  //press both together to run the self test
  if ((digitalRead(FWDButtonPin) == 0 ) && (digitalRead(REVButtonPin) == 0 )) {
    selftest();
    //I put delays in the other two butotn functions in an attempt to debounce the switch so that if you press both together it won't change the time, but it does it anyway
  }

} //end of loop		  

///////////////////////////////////////////////////////////////////////
//ChronoDot RTC stuff
void initChrono()
{
  //set_time();
  get_time();

}

void set_time()
{
  Wire.beginTransmission(104);
  Wire.write(0);
  Wire.write(decToBcd(second));
  Wire.write(decToBcd(minute));
  Wire.write(decToBcd(hh24));
  Wire.endTransmission();
}

void get_time()
{
  Wire.beginTransmission(104); 
  Wire.write(0);//set register to 0
  Wire.endTransmission();
  Wire.requestFrom(104, 3);//get 3 bytes (seconds,minutes,hours);
  second = bcdToDec(Wire.read() & 0x7f);
  minute = bcdToDec(Wire.read());
  hh24 = bcdToDec(Wire.read() & 0x3f);
}

byte decToBcd(byte val)
{
  return ( (val/10*16) + (val%10) );
}

byte bcdToDec(byte val)
{
  return ( (val/16*10) + (val%16) );
}



